import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
import * as React from "react";
import { useUid } from "./Hooks/use-uid";
import { safeBind } from "./Hooks/compose-bind";
import { useTheme } from "./Theme/Providers";
import { Text } from "./Text";
import { Touchable } from "./Touchable";
import { useMeasure } from "./Hooks/use-measure";
import { LayerRefComp } from "./Layer";
import Highlighter from "react-highlight-words";
import { usePopper } from 'react-popper';
import { BlurTriggerRefInput } from "./Form";
export const ComboBoxContext = React.createContext(null);
/** 请用ComboBoxDatalist来代替 ComboBox；
 * 虽然直接用W3C浏览器提供的<datalist标签？list做关联id,来做组合输入框，请到Form.tsx找InputDatalist组件代替本组件，
 * 但是还是这个定制的更好用。
 * ComboBox是内部使用 private的。
 * 父辈组件注入style样式，然后传递给孙子组件ComboBoxInput。
 遇到嵌套组件<ComboBox> <ComboBoxInput； 照样能够把style传递给孙子儿子ComboBoxInput；
 */
export const ComboBox = ({ children, onSelect, 
//autocomplete = false,
query, onQueryChange, onInputCommit, style }) => {
    const inputRef = React.useRef(null);
    const listRef = React.useRef(null);
    const listId = `list${useUid()}`;
    const options = React.useRef([]);
    //替换掉const position = usePositioner() of ./Hooks/use-positioner"直接使用'react-popper'包。
    const [referenceElement, setReferenceElement] = React.useState(null);
    const [popperElement, setPopperElement] = React.useState(null);
    const [arrowElement, setArrowElement] = React.useState(null);
    const { styles, attributes } = usePopper(referenceElement, popperElement, {
        modifiers: [{ name: 'arrow', options: { element: arrowElement } }],
    });
    const [expanded, setExpanded] = React.useState(false);
    const [selected, setSelected] = React.useState(null);
    const inputSize = useMeasure(inputRef);
    /**
     * Handle input change 输入框输入变更
     */
    const onInputChange = React.useCallback((e) => {
        // potentially show popover
        setSelected(null);
        if (!expanded) {
            setExpanded(true);
        }
        onQueryChange(e.target.value);
    }, [expanded, onQueryChange]);
    /**
     * Escape key pressed
     */
    const onEscape = React.useCallback(() => {
        setExpanded(false);
        setSelected(null);
    }, []);
    /**
     * Enter pressed or item clicked
     */
    const onItemSelect = React.useCallback(() => {
        // call the parent with the selected value?
        setExpanded(false);
        onSelect && onSelect(selected);
        setSelected(null);
    }, [selected, onSelect]);
    /**
     * Get the currently active option index
     * */
    const getSelectedIndex = React.useCallback(() => {
        if (!selected)
            return -1;
        return options.current.indexOf(selected || "");
    }, [options, selected]);
    /**
     * Arrow up pressed
     */
    const onArrowUp = React.useCallback(() => {
        const opts = options.current;
        const i = getSelectedIndex();
        // on input? cycle to bottom
        if (i === -1) {
            setSelected(opts[opts.length - 1]);
            // select prev
        }
        else {
            setSelected(opts[i - 1]);
        }
    }, [getSelectedIndex]);
    /**
     * Arrow down pressed
     */
    const onArrowDown = React.useCallback(() => {
        const opts = options.current;
        const i = getSelectedIndex();
        // if last, cycle to first
        if (i + 1 === opts.length) {
            setSelected(opts[0]);
            // or next
        }
        else {
            setSelected(opts[i + 1]);
        }
    }, [getSelectedIndex]);
    /**
     * Handle keydown events
     */
    const onKeyDown = React.useCallback((e) => {
        switch (e.key) {
            case "ArrowUp":
                e.preventDefault();
                onArrowUp();
                break;
            case "ArrowDown":
                e.preventDefault();
                onArrowDown();
                break;
            case "Escape":
                e.preventDefault();
                onEscape();
                break;
            case "Enter":
                e.preventDefault();
                onItemSelect();
                break;
        }
    }, [onArrowDown, onArrowUp, onEscape, onItemSelect]);
    /**
     * Handle blur events 隐藏 列表了 Selected清空；
     */
    const handleBlur = React.useCallback(() => {
        requestAnimationFrame(() => {
            const focusedElement = document.activeElement;
            const list = listRef.current;
            if (focusedElement === inputRef.current || focusedElement === list) {
                // ignore
                return;
            }
            // ignore if our popover contains the focused element
            if (list && list.contains(focusedElement)) {
                return;
            }
            // hide popover
            setExpanded(false);
            setSelected(null);
        });
    }, []);
    /**
     * handle input focus
     */
    const handleFocus = React.useCallback(() => {
        setExpanded(true);
    }, []);
    /**
     * Handle item clicks
     * ComboBoxOption列表unmounted报错{tounchable-OnEnd()}解决方式: 拖延更新  async {await setXXX}
     */
    const handleOptionSelect = React.useCallback(async (value) => {
        //onSelect 外部注入编辑setxxx()
        onSelect && onSelect(value);
        await setSelected(null);
        //setExpanded执行太快了，反而报错 useTouchable-onEnd-dispatch("RESPONDER_RELEASE")钩子，遇到unmounted了。
        //直接在前面那个setSelected 做一个await;相当于出让一个运行render机会,拖延后面这个setExpanded的运行时间点到下一个render再做，拖延更新。
        //console.log("onSelect进入 selected=",　value);
        setExpanded(false);
    }, [onSelect]);
    /**
     * Make a unique hash for list + option
     */
    const makeHash = React.useCallback((i) => {
        return listId + i;
    }, [listId]);
    //ComboBox底下的组件有 ComboBoxInput
    const focusComponents = [ComboBoxInput];
    //console.log("组合框找到focusableItems len=",focusableItems.length,"input=",focusableItems);
    return (_jsx(ComboBoxContext.Provider, Object.assign({ value: {
            listId,
            inputRef,
            listRef,
            options,
            onInputChange,
            selected,
            onKeyDown,
            handleBlur,
            handleFocus,
            handleOptionSelect,
            position: { setReferenceElement, setPopperElement, setArrowElement, styles, attributes },
            makeHash,
            expanded,
            inputSize: inputSize.bounds,
            query,
            //autocomplete,
            onInputCommit
            ///topDivStyle: style,
        } }, { children: React.Children.map(children, (child, i) => {
            if (React.isValidElement(child) &&
                focusComponents.indexOf(child.type) > -1) {
                return React.cloneElement(child, {
                    style
                });
            }
            else
                return child;
        }) }), void 0));
};
//分解配套的模式ComboBoxInput必需加到ComboBox底下的，ComboBox本身没有实际的div元素,靠context管理传递参数。
//本组件的上级直接用<ComboBoxInput css={{...style}}， 就能够传递样式到了 InputRefBase；不需要topDivStyle；
//ComboBoxDatalist情形的component必须用BlurTriggerRefInput；
export const ComboBoxInput = ({ component: Component = BlurTriggerRefInput, style, ...other }) => {
    const context = React.useContext(ComboBoxContext);
    //列表的同步 高亮提示的变化需要 query的修改内容来同步。 这个localValue实际没用到！
    //const [localValue, setLocalValue] = React.useState("");
    if (!context) {
        throw new Error("ComboBoxInput must be wrapped in a ComboBox component");
    }
    //这里topDivStyle样式从上级组件ComboBox以共享同一个context模式传递的，算是第二次传递。
    const { onKeyDown, makeHash, selected, position, handleBlur, query, 
    //autocomplete, 没啥用
    handleFocus, onInputChange, listId, inputRef, onInputCommit, listRef,
    //topDivStyle
     } = context;
    /** support autocomplete on selection */
    /*  React.useEffect(() => {
        if (!autocomplete) {
          return;
        }
        if (selected) {
          setLocalValue(selected);
        } else {
          setLocalValue("");
        }
      }, [selected, autocomplete]);*/
    //回调过滤机制
    const onNeedTrig = (e) => {
        //点击了列表项的失去焦点。
        return (!(listRef?.current && e.relatedTarget?.parentNode === listRef.current));
    };
    //设置onChange和value参数导致BlurTriggerRefInput的状态实际上每次按键立刻修改。onInputChange=>query=>所以ESC不好使;
    return (_jsx(Component, Object.assign({ id: listId, value: query, onChange: onInputChange, onSave: onInputCommit, needTrig: onNeedTrig, onBlur: handleBlur, onKeyDown: onKeyDown, onFocus: handleFocus, "aria-controls": listId, autoComplete: "off", "aria-readonly": true, "aria-autocomplete": "list", role: "textbox", css: [
            {
                ...style
            },
            // topDivStyle
        ], "aria-activedescendant": selected ? makeHash(selected) : undefined }, safeBind({
        ref: inputRef
    }, {
        ref: position.setReferenceElement
    }, other)), void 0));
};
export const ComboBoxList = ({ children, autoHide = true, ...other }) => {
    const context = React.useContext(ComboBoxContext);
    const theme = useTheme();
    if (!context) {
        throw new Error("ComboBoxInput must be wrapped in a ComboBox component");
    }
    const { inputSize, expanded, listId, handleBlur, listRef, position, options } = context;
    React.useLayoutEffect(() => {
        options.current = [];
        return () => {
            options.current = [];
        };
    });
    //不能用<Layer 组件，这个实际要算 传入ref参数给Layer的。否则报错Function components cannot be given refs.
    return (_jsx(React.Fragment, { children: (expanded || !autoHide) && (_jsxs(LayerRefComp, Object.assign({ tabIndex: -1, elevation: "sm", style: position.styles.popper, "data-placement": position.attributes.popper, id: listId, role: "listbox", onBlur: handleBlur, className: "ComboBoxList", css: {
                overflow: "hidden",
                borderRadius: theme.radii.sm,
                outline: "none",
                width: inputSize.width +
                    inputSize.left +
                    (inputSize.right - inputSize.width) +
                    "px",
                margin: 0,
                padding: 0,
                zIndex: theme.zIndices.overlay,
            } }, safeBind({
            ref: listRef
        }, {
            ref: position.setPopperElement
        }, other), { children: [children, _jsx("div", { ref: position.setArrowElement, style: position.styles.arrow }, void 0)] }), "1")) }, void 0));
};
/** ComboBox不允许列表派生参数label，字面描述就是value。
 * value才是选择输出数值， 但是ComboBoxOption组件可以内嵌子组件来表达诸如表面描述或者图片等与value有关联的东西label。
 * 但是选择了之后，在Input框显示的还是value，不能显示出label表面描述文字。
 * 类似这样用户可以修改的输入，应该不会在意关联的label表面描述文字，真的应该去掉，仅仅保留value；不像Select这样的固定的/用户无法主动输入只允许选择的才会搞出label名堂。
 * @param value
 * @param children
 * @param other
 * ComboBox不允许multiple；但是Select不允许自主添加新的列表项目，Select允许列表附加label描述文字代替value;
 */
export const ComboBoxOption = ({ value, children, ...other }) => {
    const context = React.useContext(ComboBoxContext);
    const theme = useTheme();
    if (!context) {
        throw new Error("ComboBoxInput must be wrapped in a ComboBox component");
    }
    const { makeHash, handleOptionSelect, options, selected } = context;
    React.useEffect(() => {
        if (options.current) {
            options.current.push(value);
        }
    });
    const isSelected = selected === value;
    const onClick = React.useCallback(() => {
        handleOptionSelect(value);
    }, [value, handleOptionSelect]);
    //加上terminateOnScroll={true}才能准确捕捉点击，否则容易误操作，没完全明确选定。
    //Touchable在这里实际component="li"转换为<li />标签了；这里没用到<select> <option/>
    return (_jsx(Touchable, Object.assign({ tabIndex: -1, id: makeHash(value), role: "option", component: "li", onPress: onClick, "aria-selected": isSelected ? "true" : "false", css: {
            outline: "none",
            width: "100%",
            boxSizing: "border-box",
            display: "block",
            listStyleType: "none",
            wordBreak: 'break-all',
            margin: 0,
            padding: `0.5rem 0.75rem`,
            cursor: "pointer",
            background: isSelected ? theme.colors.background.tint1 : "none",
            "&.Touchable--hover": {
                background: theme.colors.background.tint1
            },
            "&.Touchable--active": {
                background: theme.colors.background.tint2
            }
        } }, other, { children: children || _jsx(ComboBoxOptionText, { value: value }, void 0) }), void 0));
};
export const ComboBoxOptionText = ({ value, ...other }) => {
    const context = React.useContext(ComboBoxContext);
    const theme = useTheme();
    if (!context) {
        throw new Error("ComboBoxInput must be wrapped in a ComboBox component");
    }
    const { query } = context;
    return (_jsx(Text, Object.assign({ className: "ComboBoxOptionText", css: { fontWeight: theme.fontWeights.heading } }, other, { children: _jsx(Highlighter, { highlightStyle: {
                color: theme.colors.text.selected,
                background: "transparent"
            }, className: "ComboBoxOptionText__Highlighter", searchWords: [query.toString()], autoEscape: true, textToHighlight: value }, void 0) }), void 0));
};
/**类似InputDatalist组件，目的是：可以直接替换 repalce in files，能保持参数一致，移植修改简单切换。
 * 对比InputDatalist,定做组合框的版本。 而且InputDatalist弹出列表无法自己定做样式。
 * 自主输入Enter完成就变空的。 ComboBoxInput无法使用safeBind({ ref }，单独有inputRef的；
 * readOnly?: boolean; disabled?: boolean;
 * onChange不能直接利用参数类型是event：e?.currentTarget，而ComboBox这都是string类型。
 * @param onListChange:可选列表选定，或者编辑框修改,导致query={value}必然变更。
 */
export const ComboBoxDatalist = ({ autoComplete, autoFocus, inputSize = "md", fullWidth = true, datalist = [], style, value, onListChange, ...other }) => {
    //内部状态量edmatch 能避免外部频繁render; onQueryChange由input的onChange调用，每个按键点击都要触发：
    const [edmatch, setEdmatch] = React.useState(value); //仅仅value初始化用的，不会自动更新。
    React.useEffect(() => {
        //若不加上 列表框选择了 输入框就无法同步复制
        setEdmatch(value || '');
    }, [value]);
    //这一层<ComboBox 实际只是加了个 context的；
    return (_jsxs(ComboBox
    //底下的列表展开的那些，需要高亮匹配显示的，是直接context搜索{query}内容。临时编辑状态的？
    , Object.assign({ 
        //底下的列表展开的那些，需要高亮匹配显示的，是直接context搜索{query}内容。临时编辑状态的？
        query: edmatch, 
        //输入框任何修改都立刻触发的:
        onQueryChange: v => {
            setEdmatch(v);
        }, 
        //选择列表某个项目后的触发
        onSelect: v => {
            v && onListChange(v);
        }, onInputCommit: v => {
            onListChange(v);
        } }, { children: [_jsx(ComboBoxInput, Object.assign({ "aria-label": "Select add", css: {
                    ...style
                } }, other), void 0),
            _jsx(ComboBoxList, { children: datalist.length ? (datalist.map((one, i) => {
                    return _jsx(ComboBoxOption, { value: one }, i);
                })) : (_jsx("div", { children: _jsx(Text, Object.assign({ muted: true, css: { display: "block", padding: "0.5rem 0.75rem" } }, { children: value && datalist.length === 0 ? (_jsx("span", { children: "No entries found." }, void 0)) : (_jsx("span", { children: other.placeholder }, void 0)) }), void 0) }, void 0)) }, void 0)] }), void 0));
};
//# sourceMappingURL=ComboBox.js.map