import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
import * as React from "react";
import { Text } from "./Text";
import { useContext } from "react";
import { PanelEnlargeCtx } from "./TwoHalfFrame";
import { useMMenuBarOcup } from "./MainMenuBar";
import { useTheme } from "./Theme/Providers";
import { Navbar } from "./Navbar";
import { Toolbar } from "./Toolbar";
import { ResponsivePopover } from "./Popover";
import { IconButton, IconRefButton } from "./IconButton";
import { IconChevronsLeft, IconChevronsRight, IconMoreVertical } from "./Icons";
/**右半边页面框架;   目的是配合<TwoHalfFrame/>共同构建框架页面。
 * 配合使用<TwoHalfFrame/> 这两个组件的共同父辈：可能是静态路由器分割开的，那么需要在路由器的父辈，也就是index.tsx比如这样的:
 * const container = document.getElementById('root');
 * const root = createRoot(container!);
 * root.render(      底下的顶级组件注入 <XxxContext.Provider比如底下这样的：
 const [enlarge, setEnlarge] = React.useState(null);        右半边页面状态是放大吗
 return (
 <PanelEnlargeCtx.Provider  value={{ enlarge, setEnlarge }}>
 {props.children}
 </PanelEnlargeCtx.Provider>
 )}
 * */
export const TwoHalfRightPanel = ({ back, menu, title, children, ...other }) => {
    const theme = useTheme();
    const { enlarge, setEnlarge } = useContext(PanelEnlargeCtx);
    //工具条高度采用与底下的主菜单条的高度一样的设置
    const { renderList, barHeight, bottomCss } = useMMenuBarOcup(false, enlarge);
    return (_jsxs(React.Fragment, { children: [_jsx(Navbar, Object.assign({ position: "sticky", css: {
                    backgroundColor: "white",
                } }, { children: _jsxs(Toolbar, Object.assign({ css: {
                        alignItems: "center",
                        display: "flex",
                        minHeight: `${barHeight}`,
                        //底层Toolbar的设置,要覆盖掉：
                        [theme.mediaQueries.lg]: {
                            minHeight: `${barHeight}`,
                        }
                    } }, { children: [back, typeof title === "string" ? (_jsx(Text, Object.assign({ css: {
                                flex: 1,
                                textAlign: "center",
                                [theme.mediaQueries.md]: {
                                    textAlign: "left"
                                }
                            }, wrap: false, variant: "h5", gutter: false }, { children: title }), void 0))
                            :
                                (title),
                        _jsxs("div", Object.assign({ css: { display: 'inline-flex' } }, { children: [menu &&
                                    _jsx(ResponsivePopover, Object.assign({ content: menu }, { children: _jsx(IconRefButton, { css: {
                                                marginLeft: theme.spaces.sm,
                                            }, variant: "ghost", icon: _jsx(IconMoreVertical, {}, void 0), label: "\u83DC\u5355", size: "md" }, void 0) }), void 0),
                                _jsx(IconButton, { icon: enlarge ? _jsx(IconChevronsRight, {}, void 0) : _jsx(IconChevronsLeft, {}, void 0), variant: "ghost", label: "\u653E\u5927", size: "md", css: {
                                        marginRight: theme.spaces.sm,
                                        display: "none",
                                        [theme.mediaQueries.md]: {
                                            display: "flex"
                                        }
                                    }, onClick: async (e) => {
                                        await setEnlarge(!enlarge);
                                        e.stopPropagation();
                                    } }, void 0)] }), void 0)] }), void 0) }), void 0),
            _jsx("div", Object.assign({ className: "HalfPaneRM", css: [
                    {
                        display: 'flex',
                        alignItems: 'center',
                        justifyContent: 'center',
                        minHeight: '40vh',
                        [theme.mediaQueries.md]: {},
                    },
                    renderList && !enlarge ? undefined : bottomCss,
                ] }, { children: _jsx("div", Object.assign({ className: "RightToper", css: {
                        width: '100%',
                    } }, { children: children }), void 0) }), void 0)] }, void 0));
};
//# sourceMappingURL=TwoHalfRightPanel.js.map